{-# LANGUAGE OverloadedStrings #-}
module Day1.Internal (
  parseElfInput,
  Elf,
  CalorieCount,
  calories,
  elf,
  parseElfGroups,
  intOrZero,
  sortElves,
  getCalories
) where

import qualified Data.Text as T
import qualified Data.Text.Read as R
import Data.List.Split
import qualified Data.List as L
import Data.Either

newtype CalorieCount = CalorieCount Int
  deriving (Show, Eq, Ord)

newtype Elf = Elf CalorieCount
  deriving (Show, Eq, Ord)

elf :: Int -> Elf
elf c = Elf (CalorieCount c)

calories :: Int -> CalorieCount
calories = CalorieCount

getCalories :: Elf -> Int
getCalories (Elf (CalorieCount c)) = c

sortElves :: [Elf] -> [Elf]
sortElves = L.reverse . L.sort

parseElfInput :: T.Text -> [Elf]
parseElfInput input = map (elf . sum) (parseElfGroups input)

parseElfGroups :: T.Text -> [[Int]]
parseElfGroups input = map (map intOrZero) groups
  where
  groups = splitWhen (== "") $ T.lines input


intOrZero:: T.Text -> Int
intOrZero text = val
  where
  (val, _) = fromRight (0, "") $ R.decimal text

